import { useEffect, useState } from "react";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { faDrum, faGuitar, faHeadphones, faMusic } from "@fortawesome/free-solid-svg-icons";
import { useGame, useSocket } from "./context/GameContext";
import LobbyScreen from "./components/LobbyScreen";
import HomeScreen from "./components/HomeScreen";
import SongSubmissionScreen from "./components/SongSubmissionScreen";
import VotingScreen from "./components/VotingScreen";
import ResultsScreen from "./components/ResultsScreen";
import BattleResultScreen from "./components/BattleResultScreen";
import ConnectionStatus from "./components/ConnectionStatus";

const App = () => {
    const [cursorPos, setCursorPos] = useState({x: 0, y: 0});
    const { isConnected } = useSocket();
    const { lobby, error } = useGame();

    const musicNotes = [
        {id: 1, top: "8%", left: "20%", icon: faMusic, scale: 1.2},
        {id: 2, top: "75%", left: "85%", icon: faGuitar, scale: 1},
        {id: 3, top: "65%", left: "12%", icon: faHeadphones, scale: 1.1},
        {id: 4, top: "20%", left: "70%", icon: faDrum, scale: 0.9}
    ];

    useEffect(() => {
        const handleMouseMove = (e) => {
            if (!handleMouseMove.ticking) {
                handleMouseMove.ticking = true;
                requestAnimationFrame(() => {
                    setCursorPos({x: e.clientX, y: e.clientY});
                    handleMouseMove.ticking = false;
                });
            }
        };
        handleMouseMove.ticking = false;

        window.addEventListener('mousemove', handleMouseMove);
        return () => window.removeEventListener('mousemove', handleMouseMove);
    }, []);

    // Determine which screen to show based on game state
    const renderGameScreen = () => {
        if (!lobby) {
            return <HomeScreen />;
        }

        switch (lobby.state) {
            case 'LOBBY':
                return <LobbyScreen />;
            case 'SONG_SUBMISSION':
                return <SongSubmissionScreen />;
            case 'VOTING':
                return <VotingScreen />;
            case 'FINISHED':
                return <ResultsScreen />;
            case 'BATTLE':
                return <BattleResultScreen />;
            default:
                return <HomeScreen />;
        }
    };

    return (
        <>
            <div className="background-elements">
                <div className="glow-point point-1" style={{
                    transform: `translate(${(cursorPos.x / window.innerWidth - 0.5) * -20}px, ${(cursorPos.y / window.innerHeight - 0.5) * -20}px)`
                }}></div>
                <div className="glow-point point-2" style={{
                    transform: `translate(${(cursorPos.x / window.innerWidth - 0.5) * 15}px, ${(cursorPos.y / window.innerHeight - 0.5) * 15}px)`
                }}></div>
                <div className="glow-point point-3" style={{
                    transform: `translate(${(cursorPos.x / window.innerWidth - 0.5) * -10}px, ${(cursorPos.y / window.innerHeight - 0.5) * -10}px)`
                }}></div>

                {musicNotes.map((note) => (
                    <div key={`note-${note.id}`} className={`music-note note-${note.id}`}
                         style={{
                             top: note.top,
                             left: note.left,
                             fontSize: `${note.scale * 60}pt`,
                             transform: `translate(${(cursorPos.x / window.innerWidth - 0.5) * -5 * note.scale}px, ${(cursorPos.y / window.innerHeight - 0.5) * -5 * note.scale}px)`
                         }}
                    >
                        <FontAwesomeIcon icon={note.icon}/>
                    </div>
                ))}
            </div>

            <div className="content-container">
                {error && (
                    <div className="error-message">
                        <p>{error}</p>
                    </div>
                )}

                {renderGameScreen()}
                
                {/* Show connection status component */}
                <ConnectionStatus />
            </div>
        </>
    )
}

export default App;