// ConnectionStatus.jsx
import { useState, useEffect } from 'react';
import { useSocket } from '../context/GameContext';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faWifi, faExclamationTriangle, faSpinner } from '@fortawesome/free-solid-svg-icons';

function ConnectionStatus() {
  const { socket, isConnected, isReconnecting, reconnectionAttempts } = useSocket();
  const [showStatus, setShowStatus] = useState(false);
  const [offline, setOffline] = useState(!navigator.onLine);

  // Monitor browser's online/offline status
  useEffect(() => {
    const handleOnline = () => setOffline(false);
    const handleOffline = () => setOffline(true);
    
    window.addEventListener('online', handleOnline);
    window.addEventListener('offline', handleOffline);
    
    return () => {
      window.removeEventListener('online', handleOnline);
      window.removeEventListener('offline', handleOffline);
    };
  }, []);

  // Always show status when disconnected, hide after successful connection
  useEffect(() => {
    if (!isConnected || isReconnecting || offline) {
      setShowStatus(true);
    } else {
      // Hide the status indicator after a delay
      const timer = setTimeout(() => {
        setShowStatus(false);
      }, 3000);
      return () => clearTimeout(timer);
    }
  }, [isConnected, isReconnecting, offline]);

  if (!showStatus) return null;

  let statusClass = 'connection-status';
  let icon, message;

  if (offline) {
    statusClass += ' offline';
    icon = <FontAwesomeIcon icon={faExclamationTriangle} />;
    message = 'Keine Internetverbindung. Spielstand wird lokal gespeichert.';
  } else if (!isConnected) {
    statusClass += ' disconnected';
    icon = <FontAwesomeIcon icon={faExclamationTriangle} />;
    message = 'Verbindung zum Server verloren. Versuche neu zu verbinden...';
  } else if (isReconnecting) {
    statusClass += ' reconnecting';
    icon = <FontAwesomeIcon icon={faSpinner} spin />;
    message = `Verbindungsversuch ${reconnectionAttempts}...`;
  } else {
    statusClass += ' connected';
    icon = <FontAwesomeIcon icon={faWifi} />;
    message = 'Verbunden mit dem Server';
  }

  return (
    <div className={statusClass}>
      <div className="connection-icon">{icon}</div>
      <div className="connection-message">{message}</div>
    </div>
  );
}

export default ConnectionStatus;
