import { useState, useEffect, useRef } from 'react';
import { useGame } from '../context/GameContext';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faPlus, faTrash, faCheck, faMusic, faVideoCamera, faSearch, faSpinner, faExternalLinkAlt, faTimes } from '@fortawesome/free-solid-svg-icons';
import { getDisplayName } from '../utils/playerUtils';

const SongSubmissionScreen = () => {
  const { lobby, currentPlayer, addSong, removeSong, setPlayerReady, searchYouTube, getYouTubeMetadata } = useGame();
  const [songs, setSongs] = useState([]);
  const [isReady, setIsReady] = useState(false);
  const [songForm, setSongForm] = useState({
    youtubeLink: ''
  });
  const [isFormVisible, setIsFormVisible] = useState(false);
  const [searchQuery, setSearchQuery] = useState('');
  const [searchResults, setSearchResults] = useState([]);
  const [isSearching, setIsSearching] = useState(false);
  const [isLoadingMetadata, setIsLoadingMetadata] = useState(false);
  const [selectedVideo, setSelectedVideo] = useState(null);
  const [previewVisible, setPreviewVisible] = useState(false);
  const searchTimeout = useRef(null);
  
  useEffect(() => {
    if (lobby) {
      // Finde den aktuellen Spieler und seine Lieder
      const player = lobby.players.find(p => p.id === currentPlayer.id);
      if (player) {
        setIsReady(player.isReady);
      }
    }
  }, [lobby, currentPlayer]);

  // Hole die Lieder des Spielers vom Server
  useEffect(() => {
    const fetchPlayerSongs = async () => {
      if (lobby && currentPlayer) {
        console.log('Lieder für Spieler abrufen:', currentPlayer);
        console.log('Alle Spieler in der Lobby:', lobby.players.map(p => ({ id: p.id, name: p.name })));
        
        // Finde den aktuellen Spieler anhand seiner ID, Name oder Socket-ID
        let player = lobby.players.find(p => p.id === currentPlayer.id);
        
        // Falls nicht gefunden, versuche es mit dem Namen als Fallback
        if (!player) {
          player = lobby.players.find(p => p.name === currentPlayer.name);
          console.log('Spieler nicht über ID gefunden, versuche es mit Namen. Gefunden:', player);
        }
        
        // Wenn Spieler gefunden und Lieder vorhanden, aktualisiere den Zustand
        if (player) {
          console.log('Spieler gefunden:', player);
          if (player.songs && Array.isArray(player.songs)) {
            console.log('Spieler-Lieder gefunden für', player.name, ':', player.songs.length);
            console.log('Lieder-Daten:', player.songs);
            setSongs(player.songs || []);
          } else {
            console.log('Kein Lieder-Array für Spieler:', player);
            setSongs([]);
          }
        } else {
          console.error('Spieler nicht in Lobby gefunden! Aktueller Spieler:', currentPlayer.id);
          console.log('Verfügbare Spieler:', lobby.players.map(p => p.id));
          setSongs([]);
        }
      }
    };
    
    fetchPlayerSongs();
    // Wichtig: Dieser Effekt sollte ausgeführt werden, wenn sich der Lobby-Zustand ändert
  }, [lobby, currentPlayer]);

  // Extrahiere Video-ID aus YouTube-URL
  const extractVideoId = (url) => {
    if (!url) return null;
    
    const patterns = [
      /(?:https?:\/\/)?(?:www\.)?youtube\.com\/watch\?v=([^&]+)/,
      /(?:https?:\/\/)?(?:www\.)?youtube\.com\/embed\/([^/?]+)/,
      /(?:https?:\/\/)?(?:www\.)?youtu\.be\/([^/?]+)/
    ];
    
    for (const pattern of patterns) {
      const match = url.match(pattern);
      if (match && match[1]) {
        return match[1];
      }
    }
    
    return null;
  };

  // Verzögerte Suchfunktion
  const handleSearch = async (query) => {
    if (!query.trim()) {
      setSearchResults([]);
      return;
    }
    
    setIsSearching(true);
    try {
      const results = await searchYouTube(query);
      setSearchResults(results || []);
    } catch (error) {
      console.error('Suche fehlgeschlagen:', error);
    } finally {
      setIsSearching(false);
    }
  };

  const handleInputChange = async (e) => {
    const { name, value } = e.target;
    
    if (name === 'searchQuery') {
      setSearchQuery(value);
      
      // Prüfe, ob die Eingabe ein YouTube-Link sein könnte
      const videoId = extractVideoId(value);
      if (videoId) {
        setSongForm({ youtubeLink: value });
        
        // Setze grundlegende Informationen sofort für eine reaktionsschnelle Benutzeroberfläche
        const basicVideoInfo = {
          id: videoId,
          url: value,
          thumbnail: `https://img.youtube.com/vi/${videoId}/mqdefault.jpg`
        };
        setSelectedVideo(basicVideoInfo);
        setSearchResults([]); // Lösche bestehende Suchergebnisse
        
        // Hole zusätzliche Metadaten vom Server
        try {
          setIsLoadingMetadata(true);
          const metadata = await getYouTubeMetadata(videoId);
          if (metadata) {
            // Aktualisiere ausgewähltes Video mit vollständigen Metadaten
            setSelectedVideo({
              ...basicVideoInfo,
              title: metadata.title || 'Unbekannter Titel',
              artist: metadata.artist || 'Unbekannter Künstler',
              thumbnail: metadata.thumbnail || basicVideoInfo.thumbnail
            });
          }
        } catch (error) {
          console.error('Fehler beim Abrufen der Video-Metadaten:', error);
        } finally {
          setIsLoadingMetadata(false);
        }
      } else if (value.trim()) {
        // Lösche vorherigen Timeout
        if (searchTimeout.current) {
          clearTimeout(searchTimeout.current);
        }
        
        // Setze einen neuen Timeout, um zu viele API-Aufrufe zu vermeiden
        searchTimeout.current = setTimeout(() => handleSearch(value), 500);
      } else {
        setSearchResults([]);
        setSelectedVideo(null);
      }
    }
  };
  
  // Funktion zum Umschalten der Videovorschau
  const togglePreview = (result) => {
    if (selectedVideo && selectedVideo.id === result.id) {
      setPreviewVisible(!previewVisible);
    } else {
      setSelectedVideo(result);
      setPreviewVisible(true);
    }
  };

  const handleSelectSearchResult = (result) => {
    // Stelle sicher, dass wir ein vollständiges Ergebnis mit allen erforderlichen Feldern haben
    const completeResult = {
      id: result.id,
      url: result.url || `https://www.youtube.com/watch?v=${result.id}`,
      title: result.title || 'Unbekannter Titel',
      artist: result.artist || 'Unbekannter Künstler',
      thumbnail: result.thumbnail || `https://img.youtube.com/vi/${result.id}/mqdefault.jpg`
    };
    
    // Wenn ein Suchergebnis ausgewählt wird, speichere die YouTube-URL
    setSongForm({
      youtubeLink: completeResult.url
    });
    
    // Speichere das ausgewählte Video mit allen notwendigen Daten zur Übermittlung
    setSelectedVideo(completeResult);
    
    // Behalte die Suchergebnisse sichtbar, aber aktualisiere das Abfragefeld
    setSearchQuery(completeResult.title);
  };

  const handleAddSong = (e) => {
    e.preventDefault();
    
    // Verwende die Daten des ausgewählten Videos, wenn verfügbar, sonst Fallback auf Suchanfrage oder direkte Eingabe
    let songData;
    
    // Erzeuge ein konsistentes ID-Format, das später für die Löschung funktioniert
    const generateSongId = () => `song_${Date.now()}_${Math.random().toString(36).substr(2, 9)}`;
    
    if (selectedVideo) {
      // Wir haben ein ausgewähltes Video mit vollständigen Details - verwende alle verfügbaren Metadaten
      songData = {
        youtubeLink: selectedVideo.url,
        title: selectedVideo.title,
        artist: selectedVideo.artist,
        thumbnail: selectedVideo.thumbnail,
        id: generateSongId() // Konsistentes ID-Format
      };
      
      console.log("Füge Lied mit vollständigen Metadaten hinzu:", songData);
    } else {
      // Extrahiere YouTube-URL aus Suchanfrage oder direkter Eingabe
      const youtubeLink = searchQuery.trim() || songForm.youtubeLink.trim();
      if (!youtubeLink) return;
      
      // Extrahiere Video-ID, um zu überprüfen, ob es sich um einen gültigen YouTube-Link handelt
      const videoId = extractVideoId(youtubeLink);
      if (videoId) {
        // Es ist ein YouTube-Link, sende ihn zur Metadaten-Auflösung an den Server
        songData = { 
          youtubeLink: youtubeLink,
          // Füge die videoId hinzu, um die serverseitige Verarbeitung zu erleichtern
          videoId: videoId,
          id: generateSongId() // Konsistentes ID-Format
        };
        
        console.log("Füge Lied mit YouTube-Link hinzu:", songData);
      } else {
        // Kein YouTube-Link - behandle als Suchanfrage
        alert("Bitte gib einen gültigen YouTube-Link ein oder wähle ein Suchergebnis aus");
        return;
      }
    }
    
  // Füge das Lied hinzu und aktualisiere die UI, wenn der Server antwortet
  // Wir verwenden den Promise-basierten Ansatz, um auf die vom Server generierte ID für die ordnungsgemäße Löschung zu warten
  
  // Füge das Lied mit Callback hinzu, um Lieder zu aktualisieren, wenn der Server antwortet
  addSong(songData)
    .then((updatedLobby) => {
      console.log('Lied erfolgreich hinzugefügt, aktualisierte Lobby erhalten:', updatedLobby);
      if (updatedLobby && currentPlayer) {
        const player = updatedLobby.players.find(p => p.id === currentPlayer.id);
        if (player && player.songs) {
          console.log('Setze Lieder aus addSong-Antwort:', player.songs);
          setSongs([...player.songs]); // Erstelle ein neues Array, um die Zustandsaktualisierung sicherzustellen
        } else {
          console.warn('Spieler oder Lieder nicht in aktualisierter Lobby gefunden');
        }
      } else {
        console.warn('Fehlende Lobby oder currentPlayer in der Antwort');
      }
    })
    .catch(error => {
      console.error('Fehler beim Hinzufügen des Liedes:', error);
    });
  
  // Formular zurücksetzen
  setSongForm({ youtubeLink: '' });
  setSearchQuery('');
  setSearchResults([]);
  setSelectedVideo(null);
  setIsFormVisible(false);
  };

  const handleRemoveSong = (songId) => {
    removeSong(songId);
  };

  const handleSetReady = () => {
    if (songs.length === lobby.settings.songsPerPlayer) {
      setPlayerReady();
      setIsReady(true);
    }
  };

  // Prüfe, ob wir genügend Lieder eingereicht haben
  const canSubmitMoreSongs = lobby && songs.length < lobby.settings.songsPerPlayer;

  // Extrahiere YouTube-Video-ID aus verschiedenen YouTube-URL-Formaten
  const getYoutubeVideoId = (url) => {
    if (!url) return null;
    
    try {
      const urlObj = new URL(url);
      
      if (urlObj.hostname.includes('youtube.com')) {
        return new URLSearchParams(urlObj.search).get('v');
      } else if (urlObj.hostname.includes('youtu.be')) {
        return urlObj.pathname.slice(1);
      }
    } catch (e) {
      return null;
    }
    
    return null;
  };

  // Hole YouTube-Thumbnail-URL aus Video-URL
  const getYoutubeThumbnail = (url) => {
    const videoId = getYoutubeVideoId(url);
    return videoId ? `https://img.youtube.com/vi/${videoId}/mqdefault.jpg` : null;
  };

  return (
    <div className="song-submission-screen">
      <header className="screen-header">
        <h1>Füge deine Lieder hinzu</h1>
        <p className="status">
          {isReady 
            ? 'Warte auf andere Spieler...' 
            : `Füge ${lobby?.settings?.songsPerPlayer || 0} Lieder für den Kampf hinzu`}
        </p>
      </header>
      
      <div className="submission-progress">
        <div className="progress-bar">
          <div 
            className="progress-fill" 
            style={{ width: `${(songs.length / (lobby?.settings?.songsPerPlayer || 1)) * 100}%` }}
          ></div>
        </div>
        <p>{songs.length} / {lobby?.settings?.songsPerPlayer || 0} Lieder</p>
      </div>
      
      <div className="songs-list">
        {songs.map((song, index) => (
          <div key={song.id || index} className="song-card">
            <div className="song-thumbnail">
              {getYoutubeThumbnail(song.youtubeLink) ? (
                <img src={getYoutubeThumbnail(song.youtubeLink)} alt={song.title} />
              ) : (
                <div className="thumbnail-placeholder">
                  <FontAwesomeIcon icon={faMusic} />
                </div>
              )}
            </div>
            <div className="song-info">
              <h3>{song.title}</h3>
              <p>{song.artist}</p>
            </div>
            {!isReady && (
              <button className="btn icon danger" onClick={() => handleRemoveSong(song.id)}>
                <FontAwesomeIcon icon={faTrash} />
              </button>
            )}
          </div>
        ))}
        
        {canSubmitMoreSongs && !isFormVisible && !isReady && (
          <button
            className="btn add-song-btn"
            onClick={() => setIsFormVisible(true)}
          >
            <FontAwesomeIcon icon={faPlus} className="icon-margin" />
            <span>Lied hinzufügen</span>
            <span className="pixel-corner tl"></span>
            <span className="pixel-corner tr"></span>
            <span className="pixel-corner bl"></span>
            <span className="pixel-corner br"></span>
          </button>
        )}
      </div>
      
      {isFormVisible && (
        <div className="song-form-overlay">
          <form className="song-form" onSubmit={handleAddSong}>
            <h3>Lied hinzufügen</h3>
            
            <div className="form-group search-group">
            <label>Lied suchen</label>
            <div className="search-container">
              <input 
                type="text"
                name="searchQuery"
                value={searchQuery}
                onChange={handleInputChange}
                placeholder="YouTube durchsuchen oder Link einfügen..."
                className="search-input"
              />
              {isSearching && (
                <FontAwesomeIcon icon={faSpinner} className="spinner-icon" spin />
              )}
              {searchQuery && !isSearching && (
                <FontAwesomeIcon 
                  icon={faTimes} 
                  className="clear-icon" 
                  onClick={() => {
                    setSearchQuery('');
                    setSearchResults([]);
                    setSelectedVideo(null);
                    setSongForm({ youtubeLink: '' });
                  }} 
                />
              )}
            </div>
            
            {/* Zeige ausgewähltes Video ohne eingebetteten Player */}
            {selectedVideo && (
              <div className="selected-video">
                <div className="preview-header">
                  <h4>Ausgewähltes Lied</h4>
                  <a 
                    href={selectedVideo.url} 
                    target="_blank" 
                    rel="noopener noreferrer"
                    className="external-link"
                  >
                    <FontAwesomeIcon icon={faExternalLinkAlt} /> Auf YouTube ansehen
                  </a>
                </div>
                
                <div className="video-details">
                  <div className="thumbnail-container">
                    <img 
                      src={selectedVideo.thumbnail} 
                      alt={selectedVideo.title} 
                      className="selected-thumbnail" 
                    />
                    <div className="play-overlay">
                      <FontAwesomeIcon icon={faVideoCamera} />
                    </div>
                  </div>
                  <div className="selected-info">
                    <h4>{selectedVideo.title || 'Unbekannter Titel'}</h4>
                    <p>{selectedVideo.artist || 'Unbekannter Künstler'}</p>
                  </div>
                </div>
              </div>
            )}
            
            {searchResults.length > 0 && (
              <div className="search-results">
                <h4>Suchergebnisse</h4>
                {searchResults.map(result => (
                  <div 
                    key={result.id} 
                    className={`search-result ${selectedVideo && selectedVideo.id === result.id ? 'selected' : ''}`}
                    onClick={() => handleSelectSearchResult(result)}
                  >
                    <div className="result-thumbnail-container">
                      {result.thumbnail ? (
                        <img src={result.thumbnail} alt={result.title} className="result-thumbnail" />
                      ) : (
                        <div className="thumbnail-placeholder">
                          <FontAwesomeIcon icon={faMusic} />
                        </div>
                      )}
                      <div className="thumbnail-overlay">
                        <FontAwesomeIcon icon={faVideoCamera} />
                      </div>
                    </div>
                    <div className="result-info">
                      <h4>{result.title || 'Unbekannter Titel'}</h4>
                      <p>{result.artist || 'Unbekannter Künstler'}</p>
                    </div>
                  </div>
                ))}
              </div>
            )}
          </div>
          
          <div className="form-actions">
            <button type="button" className="btn secondary" onClick={() => {
              setIsFormVisible(false);
              setSearchQuery('');
              setSearchResults([]);
            }}>
              Abbrechen
            </button>
            <button 
              type="submit" 
              className="btn primary"
              disabled={!searchQuery.trim() && !songForm.youtubeLink.trim()}
            >
              Lied hinzufügen
            </button>
          </div>
        </form>
        </div>
      )}
      
      <div className="action-buttons">
        {!isReady && songs.length === lobby?.settings?.songsPerPlayer && (
          <button
            className="btn primary"
            onClick={handleSetReady}
          >
            <FontAwesomeIcon icon={faCheck} /> Kampfbereit
          </button>
        )}
      </div>
      
      {isReady && (
        <div className="waiting-message">
          <h3>Kampfbereit!</h3>
          <p>Warte auf andere Spieler, die ihre Lieder einreichen...</p>
          
          <div className="player-status">
            <h4>Spieler bereit</h4>
            <ul className="players-ready-list">
              {lobby && lobby.players.map((player, index) => (
                <li key={player.id} className={player.isReady ? 'ready' : 'not-ready'}>
                  {getDisplayName(player, lobby, currentPlayer, index)} {player.id === currentPlayer.id && lobby.settings.hidePlayerNames && '(Du)'} 
                  {player.isReady ? (
                    <FontAwesomeIcon icon={faCheck} className="ready-icon" />
                  ) : (
                    <span className="songs-count">{player.songCount}/{lobby?.settings?.songsPerPlayer}</span>
                  )}
                </li>
              ))}
            </ul>
          </div>
        </div>
      )}
    </div>
  );
};

export default SongSubmissionScreen;
