// VotingScreen.jsx
import { useState, useEffect, useMemo } from 'react';
import { useGame } from '../context/GameContext';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faVoteYea, faTrophy, faMusic, faCheck, faMedal, faCrown, faExclamationTriangle } from '@fortawesome/free-solid-svg-icons';
import YouTubeEmbed from './YouTubeEmbed';
import { getDisplayName } from '../utils/playerUtils';

function VotingScreen() {
  const { lobby, currentPlayer, submitVote, isHost, isOffline, isReconnecting } = useGame();
  const [hasVoted, setHasVoted] = useState(false);
  const [selectedSong, setSelectedSong] = useState(null);
  const [countdown, setCountdown] = useState(null);
  const [processingByeAdvance, setProcessingByeAdvance] = useState(false);
  const [offlineVoteStatus, setOfflineVoteStatus] = useState(null);
  
  // Hole aktuellen Kampf
  const battle = lobby?.currentBattle || null;

  // Berechne die Turnierphase basierend auf der Rundennummer und Gesamtliedern
  const tournamentPhase = useMemo(() => {
    if (!lobby || !battle) return '';
    
    // Hole Gesamtanzahl der Lieder im Turnier
    const totalSongs = lobby.songs?.length || 0;
    
    if (totalSongs === 0) return 'Vorrunden';
    
    // Berechne die insgesamt benötigten Runden für das Turnier
    const totalRounds = Math.ceil(Math.log2(totalSongs));
    const currentRound = battle.round + 1;
    const roundsRemaining = totalRounds - currentRound;
    
    if (roundsRemaining === 0) return 'Finale';
    if (roundsRemaining === 1) return 'Halbfinale';
    if (roundsRemaining === 2) return 'Viertelfinale';
    return 'Vorrunde';
  }, [lobby, battle]);

  // Berechne das Verhältnis der Stimmen für den Fortschrittsbalken
  const voteRatio = useMemo(() => {
    if (!battle || !hasVoted) return { song1Percent: 50, song2Percent: 50 };
    
    const totalVotes = (battle.song1Votes || 0) + (battle.song2Votes || 0);
    
    if (totalVotes === 0) return { song1Percent: 50, song2Percent: 50 };
    
    const song1Percent = Math.round((battle.song1Votes / totalVotes) * 100);
    const song2Percent = 100 - song1Percent;
    
    // Determine winner (for styling purposes)
    const winner = song1Percent > song2Percent ? 'song1' : song1Percent < song2Percent ? 'song2' : 'tie';
    
    // Calculate margin of victory for animation intensity
    const margin = Math.abs(song1Percent - song2Percent);
    const isLandslide = margin >= 30;
    
    return { song1Percent, song2Percent, winner, totalVotes, isLandslide, margin };
  }, [battle, hasVoted]);

  // Prüfe, ob der Spieler bereits abgestimmt hat
  useEffect(() => {
    if (battle && battle.votes && currentPlayer) {
      // Prüfe, ob die ID des Spielers im Stimmen-Objekt existiert
      setHasVoted(
        Object.prototype.hasOwnProperty.call(battle.votes, currentPlayer.id) || 
        battle.votes[currentPlayer.id] !== undefined
      );
    } else {
      setHasVoted(false);
    }
  }, [battle, currentPlayer]);

  // Behandle Stimmenwahl
  const handleVoteSelect = (songId) => {
    if (hasVoted) return;
    
    setSelectedSong(songId);
  };

  // Sende endgültige Stimme
  const handleSubmitVote = async () => {
    if (!selectedSong || hasVoted) return;
    
    try {
      // If offline, show status but still try to submit (it will be queued)
      if (isOffline) {
        setOfflineVoteStatus('pending');
      }
      
      await submitVote(selectedSong);
      
      if (isOffline) {
        // In offline mode, optimistically update UI
        setHasVoted(true);
        setOfflineVoteStatus('queued');
        
        // Store vote locally for later sync
        try {
          const savedVotes = JSON.parse(localStorage.getItem('pendingVotes') || '{}');
          
          if (battle) {
            savedVotes[`battle_${battle.round}_${battle.song1.id}`] = {
              songId: selectedSong,
              battleRound: battle.round,
              timestamp: Date.now()
            };
            localStorage.setItem('pendingVotes', JSON.stringify(savedVotes));
          }
        } catch (e) {
          console.error("Fehler beim Speichern der Offline-Stimme:", e);
        }
      }
      
      // setHasVoted wird jetzt durch den useEffect behandelt, der die Stimmen prüft
    } catch (error) {
      console.error("Fehler bei der Stimmabgabe:", error);
      setOfflineVoteStatus('error');
    }
  };
  
  // Handle bye round advancement - für automatisches Weiterkommen
  const handleByeAdvance = async () => {
    if (processingByeAdvance || !isHost) return;
    
    setProcessingByeAdvance(true);
    try {
      // If offline, show notification that the action will be queued
      if (isOffline) {
        setOfflineVoteStatus('byeQueued');
      }
      
      // Nur der Host kann im Bye-Modus weiterschalten
      if (battle && battle.song1 && !battle.song2 && battle.song1.id) {
        await submitVote(battle.song1.id);
      }
    } catch (error) {
      console.error("Fehler beim Fortfahren:", error);
      setOfflineVoteStatus('error');
    } finally {
      // Verzögerung, um mehrere Klicks zu verhindern
      setTimeout(() => setProcessingByeAdvance(false), 1000);
    }
  };

  // Hole YouTube-Video-IDs aus Links
  const getYouTubeId = (url) => {
    if (!url) return null;
    
    const regExp = /^.*(youtu.be\/|v\/|u\/\w\/|embed\/|watch\?v=|&v=)([^#&?]*).*/;
    const match = url.match(regExp);
    
    return (match && match[2].length === 11) ? match[2] : null;
  };

  if (!battle || !battle.song1) {
    return (
      <div className="voting-screen">
        <h2>Bereite den nächsten Kampf vor...</h2>
      </div>
    );
  }
  
  // Behandle "Freilos"-Runden, in denen ein Lied automatisch weiterkommt
  if (battle.bye === true && battle.song1 && !battle.song2) {
    const song1Id = getYouTubeId(battle.song1?.youtubeLink || '');
    
    return (
      <div className="voting-screen">
        <header className="voting-header">
          <h1>
            <FontAwesomeIcon icon={faTrophy} /> Automatisches Weiterkommen
          </h1>
          <div className="round-info">
            <span>Runde {battle.round + 1}</span>
          </div>
        </header>
        
        <div className="bye-container">
          <div className="song-card bye-winner">
            <div className="song-spotlight"></div>
            <div className="song-details">
              <h3>{battle.song1.title}</h3>
              <p>{battle.song1.artist}</p>
              <div className="auto-advance-notice">
                <p>Dieses Lied kommt automatisch in die nächste Runde!</p>
                <p className="host-info">{isHost ? 'Du kannst als Host zum nächsten Kampf weitergehen.' : 'Warte auf den Host, um fortzufahren.'}</p>
              </div>
            </div>
            
            {song1Id ? (
              <div className="video-container">
                <YouTubeEmbed videoId={song1Id} />
              </div>
            ) : (
              <div className="no-video">
                <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
                <span>Kein Video verfügbar</span>
              </div>
            )}
          </div>
        </div>
        
        <div className="voting-status">
          {isHost ? (
            <button 
              className={`btn primary pixelated full-width ${processingByeAdvance ? 'disabled' : ''}`} 
              onClick={handleByeAdvance}
              disabled={processingByeAdvance}
            >
              {processingByeAdvance ? 'Wird geladen...' : 'Mach weiter'}
              <span className="pixel-corner tl"></span>
              <span className="pixel-corner tr"></span>
              <span className="pixel-corner bl"></span>
              <span className="pixel-corner br"></span>
            </button>
          ) : (
            <p className="auto-advance-notice">Warte auf den Host, um fortzufahren...</p>
          )}
        </div>
      </div>
    );
  }

  const song1Id = getYouTubeId(battle.song1?.youtubeLink || '');
  const song2Id = getYouTubeId(battle.song2?.youtubeLink || '');

  return (
    <div className="voting-screen">
      <header className="voting-header">
        <h1>
          <FontAwesomeIcon icon={tournamentPhase === 'Finale' ? faCrown : tournamentPhase === 'Halbfinale' ? faMedal : faVoteYea} /> 
          {tournamentPhase}
        </h1>
        <div className="round-info">
          <span>Runde {battle.round + 1}</span>
          {hasVoted && <span className="voted-badge">Du hast abgestimmt!</span>}
        </div>
      </header>
      
      <div className="battle-container">
        <div 
          className={`song-card ${selectedSong === battle.song1.id ? 'selected' : ''} ${hasVoted ? 'voted' : ''}`}
          onClick={() => handleVoteSelect(battle.song1.id)}
        >
          <div className="song-spotlight"></div>
          <div className="song-details">
            <h3>{battle.song1.title}</h3>
            <p>{battle.song1.artist}</p>
          </div>
          
          {song1Id ? (
            <div className="video-container">
              <YouTubeEmbed videoId={song1Id} />
              <div className="video-overlay"></div>
            </div>
          ) : (
            <div className="no-video">
              <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
              <span>Kein Video verfügbar</span>
            </div>
          )}
          
          {hasVoted && (
            <div className="vote-count">
              <span className="vote-number">{battle.song1Votes}</span>
              <span className="vote-text">Stimmen</span>
            </div>
          )}
          
          {selectedSong === battle.song1.id && !hasVoted && (
            <div className="selection-indicator">
              <FontAwesomeIcon icon={faCheck} className="check-icon" />
            </div>
          )}
        </div>
        
        <div className="versus">
          <span className="versus-text">VS</span>
          <div className="versus-decoration"></div>
        </div>
        
        <div 
          className={`song-card ${selectedSong === battle.song2.id ? 'selected' : ''} ${hasVoted ? 'voted' : ''}`}
          onClick={() => handleVoteSelect(battle.song2.id)}
        >
          <div className="song-spotlight"></div>
          <div className="song-details">
            <h3>{battle.song2.title}</h3>
            <p>{battle.song2.artist}</p>
          </div>
          
          {song2Id ? (
            <div className="video-container">
              <YouTubeEmbed videoId={song2Id} />
              <div className="video-overlay"></div>
            </div>
          ) : (
            <div className="no-video">
              <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
              <span>Kein Video verfügbar</span>
            </div>
          )}
          
          {hasVoted && (
            <div className="vote-count">
              <span className="vote-number">{battle.song2Votes}</span>
              <span className="vote-text">Stimmen</span>
            </div>
          )}
          
          {selectedSong === battle.song2.id && !hasVoted && (
            <div className="selection-indicator">
              <FontAwesomeIcon icon={faCheck} className="check-icon" />
            </div>
          )}
        </div>
      </div>
      
      {/* Vote Progress Bar - Only show after voting */}
      {hasVoted && (
        <div className="vote-progress-container">
          <div className="vote-progress-labels">
            <span className={`vote-label left ${voteRatio.winner === 'song1' ? 'winning' : ''} ${voteRatio.winner === 'song1' && voteRatio.isLandslide ? 'landslide' : ''}`}>
              {battle.song1Votes || 0} Stimmen
              {voteRatio.winner === 'song1' && voteRatio.totalVotes > 1 && 
                <span className="winner-crown"><FontAwesomeIcon icon={faCrown} /></span>
              }
            </span>
            <span className={`vote-label right ${voteRatio.winner === 'song2' ? 'winning' : ''} ${voteRatio.winner === 'song2' && voteRatio.isLandslide ? 'landslide' : ''}`}>
              {battle.song2Votes || 0} Stimmen
              {voteRatio.winner === 'song2' && voteRatio.totalVotes > 1 && 
                <span className="winner-crown"><FontAwesomeIcon icon={faCrown} /></span>
              }
            </span>
          </div>
          <div className={`vote-progress-bar ${voteRatio.winner !== 'tie' ? voteRatio.winner : ''} ${voteRatio.isLandslide ? 'landslide' : ''}`}>
            <div 
              className={`vote-progress-fill song1 ${voteRatio.winner === 'song1' ? 'winning' : ''}`}
              style={{ width: `${voteRatio.song1Percent}%` }}
            >
              {voteRatio.song1Percent > 5 && (
                <div className="vote-progress-decoration left"></div>
              )}
            </div>
            <div 
              className={`vote-progress-divider ${voteRatio.margin <= 10 ? 'close-race' : ''}`}
              style={{ left: `${voteRatio.song1Percent}%` }}
            ></div>
            <div 
              className={`vote-progress-fill song2 ${voteRatio.winner === 'song2' ? 'winning' : ''}`}
              style={{ width: `${voteRatio.song2Percent}%` }}
            >
              {voteRatio.song2Percent > 5 && (
                <div className="vote-progress-decoration right"></div>
              )}
            </div>
          </div>
          <div className="vote-progress-percentages">
            <span className={`vote-percent left ${voteRatio.winner === 'song1' ? 'winning' : ''} ${voteRatio.winner === 'song1' && voteRatio.isLandslide ? 'landslide' : ''}`}>
              {voteRatio.song1Percent}%
            </span>
            <span className={`vote-percent right ${voteRatio.winner === 'song2' ? 'winning' : ''} ${voteRatio.winner === 'song2' && voteRatio.isLandslide ? 'landslide' : ''}`}>
              {voteRatio.song2Percent}%
            </span>
          </div>
        </div>
      )}
      
      {!hasVoted && (
        <div className="voting-actions">
          <button 
            className={`btn primary ${isOffline ? 'offline' : ''}`} 
            onClick={handleSubmitVote}
            disabled={!selectedSong}
          >
            {isOffline ? 'Offline Abstimmen' : 'Abstimmen'}
          </button>
          
          {isOffline && (
            <div className="offline-notice">
              <FontAwesomeIcon icon={faExclamationTriangle} />
              <span>Deine Stimme wird gespeichert und gesendet, sobald die Verbindung wiederhergestellt ist.</span>
            </div>
          )}
          
          {offlineVoteStatus === 'queued' && (
            <div className="offline-vote-status">
              <span>Stimme gespeichert! Wird gesendet, wenn online.</span>
            </div>
          )}
          
          {offlineVoteStatus === 'error' && (
            <div className="offline-vote-status error">
              <span>Fehler beim Speichern der Stimme.</span>
            </div>
          )}
        </div>
      )}
      
      <div className="voting-status">
        <p>{hasVoted ? 'Warte auf andere Spieler...' : 'Wähle deinen Favoriten!'}</p>
        
        {isReconnecting && (
          <div className="reconnecting-notice">
            <span>Versuche die Verbindung wiederherzustellen...</span>
          </div>
        )}
        
        <div className="votes-count">
          <span>{battle.voteCount || 0}</span> von <span>{lobby?.players?.filter(p => p.isConnected).length || 0}</span> Stimmen
          {isOffline && <span className="offline-badge"> (Offline-Modus)</span>}
        </div>
        
        {/* Liste der Spielerstimmen */}
        <div className="player-votes">
          <h4>Abstimmende</h4>
          <ul className="players-voted-list">
            {lobby?.players?.filter(p => p.isConnected).map(player => {
              // Prüfe, ob dieser Spieler abgestimmt hat
              const hasPlayerVoted = battle.votes && 
                Object.keys(battle.votes).includes(player.id);
              
              // Highlight current player to show they've voted offline
              const isCurrentPlayerOfflineVoted = player.id === currentPlayer.id && 
                                                isOffline && 
                                                hasVoted;
              
              return (
                <li key={player.id} className={`${hasPlayerVoted ? 'voted' : 'not-voted'} ${isCurrentPlayerOfflineVoted ? 'offline-voted' : ''}`}>
                  {getDisplayName(player, lobby, currentPlayer)} {player.id === currentPlayer.id && lobby.settings.hidePlayerNames && '(Du)'}
                  {hasPlayerVoted && 
                    <FontAwesomeIcon icon={faCheck} className="vote-icon" />
                  }
                  {isCurrentPlayerOfflineVoted && 
                    <FontAwesomeIcon icon={faExclamationTriangle} className="offline-icon" />
                  }
                </li>
              );
            })}
          </ul>
        </div>
      </div>
    </div>
  );
}

export default VotingScreen;
