/**
 * Utility functions for player name handling
 */

/**
 * Returns an anonymous name for a player when hidePlayerNames is enabled
 * @param {string} playerId - The player's ID
 * @param {number} index - The player's index in a list (optional)
 * @param {boolean} isCurrentPlayer - Whether this is the current player
 * @returns {string} - Anonymous name or indicator
 */
export const getAnonymousName = (playerId, index = null, isCurrentPlayer = false) => {
  if (isCurrentPlayer) {
    return 'Du';
  }
  
  // Use index-based naming if provided
  if (index !== null) {
    return `Spieler ${index + 1}`;
  }
  
  // Use last 4 characters of ID as fallback
  const shortId = playerId.slice(-4);
  return `Spieler ${shortId}`;
};

/**
 * Returns the appropriate player name based on game settings
 * @param {Object} player - The player object
 * @param {Object} lobby - The lobby object
 * @param {Object} currentPlayer - The current player
 * @param {number} index - The player's index in a list (optional)
 * @returns {string} - The player name or anonymous identifier
 */
export const getDisplayName = (player, lobby, currentPlayer, index = null) => {
  const isCurrentPlayer = player.id === currentPlayer?.id;
  
  // If hidePlayerNames is enabled
  if (lobby?.settings?.hidePlayerNames) {
    return getAnonymousName(player.id, index, isCurrentPlayer);
  }
  
  // Otherwise return the actual name
  return player.name;
};
